/*** ISOLADO.C -- Verifica a existencia de pontos isolados na intersecao ***/

/* Constantes usadas no calculo de pontos isolados */
double A_ERRO = 0.05;
int I_QUANT = 1;
double I_ERRO = 0.05, I_INCR = 0.01, I_RAIO = 0.05;
char ARQ_ISOLADOS[30] = "isolados.txt";

/* ----------------------------------------------------------------------- */

void SalvaPontoIsolado(double Px, double Py, double Pz,
				double pu, double pv, double qu, double qv) {

  FILE *arq;

  if ((arq = fopen(ARQ_ISOLADOS, "wt")) == NULL) return;

  fprintf(arq, "\nPonto isolado na intersecao de F e G\n");
  fprintf(arq, "\n(%g, %g, %g) = F(%g, %g) = G(%g, %g)\n",
				  Px, Py, Pz, pu, pv, qu, qv);
  fclose(arq);

  fprintf(arq_padrao, "\n(%0.*f %0.*f %0.*f)",
			 ARQ_DEC, Px, ARQ_DEC, Py, ARQ_DEC, Pz);
  if (ARQ_DOMINIO) {
      fprintf(arq_padrao, " (%0.*f %0.*f)",
			 ARQ_DEC, pu, ARQ_DEC, pv);
      fprintf(arq_padrao, " (%0.*f %0.*f)",
			 ARQ_DEC, qu, ARQ_DEC, qv);
  }
  if (ARQ_VETORES)
      fprintf(arq_padrao, " (0.0 0.0 0.0)");
  if (ARQ_CIRC)
      fprintf(arq_padrao, " {(0.0 0.0 0.0) 0.0}");
  if (ITERACOES)
      fprintf(arq_padrao, " [0]");
}

/* ----------------------------------------------------------------------- */

double Arredonda(double x) {

/* Arredonda x para o inteiro mais proximo ou para um multiplo de Pi/4
   mais proximo. E' usado um erro A_ERRO na identificacao. */

  double xcopia;

  if (fabs(ceil(x) - x) < A_ERRO) return ceil(x);
  if (fabs(floor(x) - x) < A_ERRO) return floor(x);

  if (fabs(x - PI) < A_ERRO) return PI;
  if (fabs(x - PI_SOBRE_2) < A_ERRO) return PI_SOBRE_2;
  if (fabs(x - DOIS_PI) < A_ERRO)  return DOIS_PI;
  if (fabs(x + PI) < A_ERRO) return -PI;
  if (fabs(x + PI_SOBRE_2) < A_ERRO) return -PI_SOBRE_2;
  if (fabs(x + DOIS_PI) < A_ERRO)  return -DOIS_PI;

  xcopia = x/PI;
  if (fabs(ceil(xcopia) - xcopia) < A_ERRO) return PI*ceil(xcopia);
  if (fabs(floor(xcopia) - xcopia) < A_ERRO) return PI*floor(xcopia);
  xcopia = x/PI_SOBRE_2;
  if (fabs(ceil(xcopia) - xcopia) < A_ERRO) return PI_SOBRE_2*ceil(xcopia);
  if (fabs(floor(xcopia) - xcopia) < A_ERRO) return PI_SOBRE_2*floor(xcopia);
  xcopia = x/PI_SOBRE_4;
  if (fabs(ceil(xcopia) - xcopia) < A_ERRO) return PI_SOBRE_4*ceil(xcopia);
  if (fabs(floor(xcopia) - xcopia) < A_ERRO) return PI_SOBRE_4*floor(xcopia);

  return x;
}

/* ----------------------------------------------------------------------- */

int VerificaPontoIsolado(double u1, double v1, double (*f1)(double, double),
	   double (*f2)(double, double), double (*f3)(double, double),
	   double u2, double v2, double (*g1)(double, double),
	   double (*g2)(double, double), double (*g3)(double, double),
	   double raio, double incr) {

/* Verifica se um ponto isolado pertence `a intersecao. Retorna 1 somente
   se pertencer. */

  double d, uf, vf, ug, vg;
  static int quant = 0;
  static int calcular_ponto_isolado = 1;

  if (!calcular_ponto_isolado) return 0;

  uf = Arredonda(u1), ug = Arredonda(u2),
  vf = Arredonda(v1), vg = Arredonda(v2);

  d = DISTANCIA(f1(uf, vf), f2(uf, vf), f3(uf, vf),
		  g1(ug, vg), g2(ug, vg), g3(ug, vg));

  if (d < I_ERRO) {
    quant++;
    if (quant >= I_QUANT) calcular_ponto_isolado = 0;
    SalvaPontoIsolado(f1(uf, vf), f2(uf, vf), f3(uf, vf),
						    uf, vf, ug, vg);
    return 1;
  }

  for (vg = v2 - raio; vg <= v2 + raio; vg += incr)
    for (ug = u2 - raio; ug <= u2 + raio; ug += incr)
      for (vf = v1 - raio; vf <= v1 + raio; vf += incr)
	for (uf = u1 - raio; uf <= u1 + raio; uf += incr) {
	  d = DISTANCIA(f1(uf, vf), f2(uf, vf), f3(uf, vf),
				      g1(ug, vg), g2(ug, vg), g3(ug, vg));
	  if (d < I_ERRO) {
	    quant++;
	    if (quant >= I_QUANT) calcular_ponto_isolado = 0;
	    SalvaPontoIsolado(f1(uf, vf), f2(uf, vf), f3(uf, vf),
						    uf, vf, ug, vg);
	    return 1;
	  }
	}

  return 0;
}

/* ----------------------------------------------------------------------- */

/*** FIM DE "ISOLADO.C" ***/
